TEST_MODE=False
import os
import sys
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtutils import jgtconstants as constants
#import jgtfxcommon as jgtcommon
from jgtutils import jgtos,jgtcommon
import argparse

import JGTPDS as pds

#from JGTPDS import getPH as get_price, stayConnectedSetter as set_stay_connected, disconnect,connect as on,disconnect as off, status as connection_status,  getPH2file as get_price_to_file, stayConnectedSetter as sc,getPH as ph,getPH_to_filestore as ph2fs

import pandas as pd

def parse_args():
    parser = argparse.ArgumentParser(description='Process command parameters.')
    #jgtfxcommon.add_main_arguments(parser)
    jgtcommon.add_instrument_timeframe_arguments(parser)
    #jgtfxcommon.add_date_arguments(parser)
    jgtcommon.add_tlid_range_argument(parser)
    jgtcommon.add_max_bars_arguments(parser)
    jgtcommon.add_viewpath_argument(parser)
    jgtcommon.add_exit_if_error(parser)
    #jgtfxcommon.add_output_argument(parser)
    jgtcommon.add_compressed_argument(parser)
    #jgtfxcommon.add_quiet_argument(parser)
    jgtcommon.add_verbose_argument(parser)
    jgtcommon.add_debug_argument(parser)
    #jgtfxcommon.add_cds_argument(parser)
    jgtcommon.add_iprop_init_argument(parser)
    jgtcommon.add_pdsserver_argument(parser)
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    exit_on_error = False
    if args.exitonerror:
        exit_on_error = True
    
    instrument = args.instrument
    timeframe = args.timeframe
    quotes_count = -1
    using_tlid = False
    tlid_range= None
    viewpath = args.viewpath
    date_from = None
    date_to = None
    if args.tlidrange is not None:
        using_tlid= True
        tlid_range = args.tlidrange
        #print(tlid_range)
        #dtf,dtt = jgtfxcommon.tlid_range_to_start_end_datetime(tlid_range)
        #print(str(dtf) + " " + str(dtt))
        #date_from =dtf
        #date_to = dtt
    
    quotes_count = args.quotescount
    #print(args.quotescount)
    debug = args.debug
    if args.server == True:
        try:
            from . import pdsserver as svr
            svr.app.run(debug=debug)
        except:
            print("Error starting server")
            return
    if args.iprop == True:
        try:
            from . import dl_properties
            print("--------------------------------------------------")
            print("------Iprop should be downloaded in $HOME/.jgt---")
            return # we quit
        except:
            print("---BAHHHHHHHHHH Iprop trouble downloading-----")
            return
        


    
    compress=False
    verbose_level = args.verbose
    quiet=False
    output = True   # We always output
    if verbose_level == 0:
        quiet=True
    #print("Verbose level : " + str(verbose_level))

    if args.compress:
        compress = args.compress
        

    

    try:
        
        print_quiet(quiet,"Getting for : " + instrument + "_" + timeframe)
        instruments = instrument.split(',')
        timeframes = timeframe.split(',')


        if not viewpath:pds.stayConnectedSetter(True)
        
        for instrument in instruments:
            for timeframe in timeframes:
                
                if not viewpath:
                    #print("---------DEBUG jgtfxcli ------")
                    if quotes_count==-1:
                        fpath,df = pds.getPH2file(instrument, timeframe, quotes_count, None, None, False, quiet, compress,tlid_range=tlid_range)
                    else:
                        #we will try to call with an end date from tlid and a count (so we would have only an end date)
                        start_date = None;end_date = None
                        try: start_date,end_date = jgtos.tlid_range_to_start_end_datetime(tlid_range)
                        except:pass
                        if TEST_MODE:
                            print("----------TEST_MODE--------")
                            print("start_date : " + str(start_date))
                            print("end_date : " + str(end_date))
                        try:
                            fpath,df = pds.getPH2file(instrument, timeframe, quotes_count, None, end_date, False, quiet, compress)
                            print_quiet(quiet, fpath)
                        except Exception as e:
                            error_message = f"An error occurred with {instrument} {timeframe}: {e}"
                            if exit_on_error:
                                print_quiet(quiet,error_message)
                                
                                sys.exit(1)
                            else:
                                print("Failed getting:" + instrument + "_" + timeframe)
                        if TEST_MODE:
                            print(df.head(1))
                            print(df.tail(1))
                            df.to_csv("test.csv")
                        
                else:
                    fpath = pds.create_filestore_path(instrument, timeframe, quiet, compress, tlid_range, None, "pds")
                    print(fpath)
                        #pds.mk_fullpath(instrument, timeframe, tlid_range=tlid_range)

        if not viewpath:pds.disconnect()  
    except Exception as e:
        jgtcommon.print_exception(e)

    try:
        pds.disconnect()
    except Exception as e:
        jgtcommon.print_exception(e)

# print("")
# #input("Done! Press enter key to exit\n")




def print_quiet(quiet,content):
    if not quiet:
        print(content)


if __name__ == "__main__":
    main()



#%% imports
from jgtfxc import get_price_history
import jgtfxc as jfx
from datetime import datetime
import JGTPDS as pds
#%% Define the parameters
instrument = "EUR/USD"
timeframe = "H4"
#datefrom = datetime(2021, 1, 1)
datefrom = None
dateto = datetime(2021, 12, 31)
quotes_count_spec = 400
quiet = True

#%% PDS

pds.stayConnectedSetter(True)
df=pds.getPH(instrument, timeframe, datefrom, dateto, quotes_count_spec, quiet)
pds.disconnect()

#%% Call the function
jfx.stayConnected = True
jfx.connect()
jfx.get_price_history(instrument, timeframe, datefrom, dateto, quotes_count_spec, quiet)
jfx.disconnect()

# %%



#%% Imports

# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import sys
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import pandas as pd
from forexconnect import ForexConnect, fxcorepy
#from jgtpy import JGTConfig as conf

from dotenv import load_dotenv
from dotenv import dotenv_values

load_dotenv()  # take environment variables from .env.
#env=load_dotenv(os.getenv(os.getcwd()))
env = dotenv_values(".env")

if os.getenv('user_id') == "":
  load_dotenv(os.getenv('HOME'))
if os.getenv('user_id') == "":
  load_dotenv(os.getenv(os.getcwd()))

user_id = os.getenv('user_id')
password = os.getenv('password')
url = os.getenv('url')
connection = os.getenv('connection')
quotes_count = os.getenv('quotes_count')

#from jgtpy import jgtcommon as jgtcomm,iprops
from jgtutils import jgtcommon as jgtcomm,iprops

##import jgtcommon as jgtcomm,iprops

import common_samples
"""

url='https://www.fxcorporate.com/Hosts.jsp'
connection='Demo'
quotes_count='800'
"""

def parse_args():
    parser = argparse.ArgumentParser(description='Process command parameters.')
    #jgtcomm.add_main_arguments(parser)
    jgtcomm.add_instrument_timeframe_arguments(parser)
    #common_samples.add_date_arguments(parser)
    jgtcomm.add_tlid_range_argument(parser)
    #jgtcomm.add_date_arguments(parser)
    jgtcomm.add_max_bars_arguments(parser)
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    str_user_id = user_id#args.l
    str_password = password#args.p
    str_url = url#args.u
    using_tlid = False
    if args.tlidrange is not None:
      using_tlid= True
      tlid_range = args.tlidrange
      #print(tlid_range)
      dtf,dtt = jgtcomm.tlid_range_to_start_end_datetime(tlid_range)
      #print(str(dtf) + " " + str(dtt))
      date_from =dtf
      date_to = dtt
      print(str(date_from),str(date_to))
    else:
      quotes_count = args.quotescount
      
    str_connection = connection#args.c
    #str_session_id = args.session
    #str_pin = args.pin
    
    str_instrument = args.instrument
    str_timeframe = args.timeframe
    

    ip = iprops.get_iprop(instrument=str_instrument)
    pips= ip["pips"]
    lpip=len(str(pips))
    
    with ForexConnect() as fx:
        try:
            fx.login(str_user_id, str_password, str_url,
                     str_connection,
                     common_samples.session_status_changed)

            #print("")
            #print("Requesting a price history...")
            if using_tlid:
              history = fx.get_history(str_instrument, str_timeframe, date_from, date_to)
            else:
              history = fx.get_history(str_instrument, str_timeframe,None,None, quotes_count)
            current_unit, _ = ForexConnect.parse_timeframe(str_timeframe)
           
            #date_format = '%m.%d.%Y %H:%M:%S'
            date_format = '%Y-%m-%d %H:%M:%S'
            if current_unit == fxcorepy.O2GTimeFrameUnit.TICK:
                print("Date, Bid, Ask")
                #print(history.dtype.names)
                for row in history:
                    print("{0:s}, {1:,.5f}, {2:,.5f}".format(
                        pd.to_datetime(str(row['Date'])).strftime(date_format), row['Bid'], row['Ask']))
            else:
                print("Date, Open, High, Low, Close, Median, Volume")
                rounder = lpip+1
                for row in history:
                    print(format_output(rounder,row,rounder,date_format))
                    
                    # print("{0:s},{1:.5f},{2:.5f},{3:.5f},{4:.5f},{5:.5f},{6:d}".format(                    
                    #     pd.to_datetime(str(row['Date'])).strftime(date_format), open_price, high_price,
                    #     low_price, close_price, median, row['Volume']))
        except Exception as e:
            jgtcomm.print_exception(e)
        try:
            fx.logout()
        except Exception as e:
            jgtcomm.print_exception(e)

def format_output(nb_decimal, row, rounder, date_format = '%Y-%m-%d %H:%M:%S'):
    open_price = round(((row['BidOpen'] + row['AskOpen']) / 2), rounder)
    high_price = round(((row['BidHigh'] + row['AskHigh']) / 2), rounder)
    low_price = round(((row['BidLow'] + row['AskLow']) / 2), rounder)
    close_price = round(((row['BidClose'] + row['AskClose']) / 2), rounder)
    median = round(((high_price + low_price) / 2), rounder)
    dt_formatted=pd.to_datetime(str(row['Date'])).strftime(date_format)
    #print("dt formatted: " + dt_formatted)
    formatted_string = f"{dt_formatted},{open_price:.{nb_decimal}f},{high_price:.{nb_decimal}f},{low_price:.{nb_decimal}f},{close_price:.{nb_decimal}f},{median:.{nb_decimal}f},{row['Volume']:d}"
    return formatted_string
  
def format_output1(nb_decimal, row, rounder,date_format = '%Y-%m-%d %H:%M:%S'):
    open_price = round(((row['BidOpen'] + row['AskOpen']) / 2),rounder)
    high_price = round(((row['BidHigh'] + row['AskHigh']) / 2),rounder)
    low_price = round(((row['BidLow'] + row['AskLow']) / 2),rounder)
    close_price = round(((row['BidClose'] + row['AskClose']) / 2),rounder)
    median = round(((high_price + low_price) / 2),rounder)
    format_specifier = f"{{:.{nb_decimal}f}}"
    formatted_string = f"{pd.to_datetime(str(row['Date'])).strftime(date_format)},{open_price:{format_specifier}},{high_price:{format_specifier}},{low_price:{format_specifier}},{close_price:{format_specifier}},{median:{format_specifier}},{row['Volume']:d}"
    return formatted_string

# Usage

if __name__ == "__main__":
    main()
    print("")
    #input("Done! Press enter key to exit\n")



import datetime
import unittest

import os
import sys
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtfxc import parse_date

class TestParseDate(unittest.TestCase):
    def test_parse_date(self):
        # Test case 1: Valid date format '%d.%m.%Y %H:%M:%S'
        date_str = "12.31.2022 23:59:59"
        expected_result = datetime.datetime(2022, 12, 31, 23, 59, 59)
        parsed = parse_date(date_str)
        self.assertEqual(parsed, expected_result)

        # Test case 2: Valid date format '%d.%m.%Y %H:%M'
        date_str = "01.01.2023 00:00"
        expected_result = datetime.datetime(2023, 1, 1, 0, 0)
        self.assertEqual(parse_date(date_str), expected_result)

        # Test case 3: Valid date format '%d.%m.%Y'
        date_str = "01.02.2023"
        expected_result = datetime.datetime(2023, 1, 2)
        self.assertEqual(parse_date(date_str), expected_result)

        # Test case 4: Valid date format '%Y%m%d%H%M'
        date_str = "202301020000"
        expected_result = datetime.datetime(2023, 1, 2, 0, 0)
        self.assertEqual(parse_date(date_str), expected_result)

        # Test case 5: Valid date format '%y%m%d%H%M'
        date_str = "2301020000"
        expected_result = datetime.datetime(2023, 1, 2, 0, 0)
        self.assertEqual(parse_date(date_str), expected_result)

        # Test case 6: Valid date format '%Y-%m-%d %H:%M'
        date_str = "2023-01-02 00:00"
        expected_result = datetime.datetime(2023, 1, 2, 0, 0)
        self.assertEqual(parse_date(date_str), expected_result)

        # Test case 7: Invalid date format
        date_str = "2023/01/02"
        with self.assertRaises(ValueError):
            parse_date(date_str)

        print("All test cases passed!")

if __name__ == "__main__":
    unittest.main()